from django.shortcuts import render
from django.shortcuts import redirect

from vehicles.models import Vehicle, populate_vehicles_from_csv_string
from vehicles.forms import BulkCreateVehiclesForm

def vehicle(request, pk):
    vehicle = Vehicle.objects.get(pk=pk)
    context = {
        "vehicle": vehicle
    }
    return render(request, "vehicle.html", context)


def vehicles(request):
    vehicles = Vehicle.objects.all()

    if vehicles:
        context = {
            "vehicles": vehicles
        }

        return render(request, "vehicles.html", context)

    else:
        return render(request, "bulk_create_vehicles.html", {})

def bulk_create_vehicles(request):
    if request.method == "POST":
        form = BulkCreateVehiclesForm(request.POST)
        csv_data = form["csv_data"].value()
        populate_vehicles_from_csv_string(csv_data)
        return(redirect("vehicles"))

    return render(request, "bulk_create_vehicles.html", {})



from datetime import datetime, timedelta

from django.shortcuts import render
from django.shortcuts import redirect

from salaora import now_cleaned

from reservations.models import Reservation
from reservations.forms import ReservationForm

from vehicles.models import Vehicle

def crud_reservation(request, pk=0):

    if request.method == "POST":

        reservation = None
        form = ReservationForm(request.POST)
        vehicle = Vehicle.objects.get(pk=form["vehicle"].value())
        client_name = form["client_name"].value()
        departure_dt = form["departure_dt"].value()
        return_dt = form["return_dt"].value()
        details = form["details"].value()

        if pk:
            try:
                reservation = Reservation.objects.get(pk=pk)
                reservation.vehicle = vehicle
                reservation.client_name = client_name
                reservation.departure_dt = departure_dt
                reservation.return_dt = return_dt
                reservation.details = details
                reservation.save()
            except Reservation.DoesNotExist:
                pass
        else:
            reservation = Reservation(
                vehicle = vehicle,
                client_name = client_name,
                departure_dt = departure_dt,
                return_dt = return_dt,
                details = details
            )
            reservation.save()

        return redirect("reservations")

    # request.method == "GET"
    else:
        reservation = None
        update_flag = False

        if pk:
            try:
                reservation = Reservation.objects.get(pk=pk)
                update_flag = True
            except Reservation.DoesNotExist:
                # Redirect to create a new one
                return redirect("crud_reservation")

        vehicles = [vehicle for vehicle in Vehicle.objects.all()]

        default_departure_dt = datetime.now().replace(hour=9, minute=0, second=0, microsecond=0)
        default_return_dt = datetime.now().replace(hour=21, minute=0, second=0, microsecond=0)

        context = {
            "reservation": reservation,
            "update_flag": update_flag,
            "vehicles": vehicles,
            "default_departure_dt": default_departure_dt,
            "default_return_dt": default_return_dt
        }

    return render(request, "crud_reservation.html", context)

def delete_reservation(request, pk):
    try:
        reservation = Reservation.objects.get(pk=pk)
        reservation.delete()
    except Reservation.DoesNotExist:
        pass
        # Redirect to create a new one
    return redirect("reservations")

def reservations(request):
    reservations = Reservation.objects.all()
    context = {
        "reservations": reservations
    }

    return render(request, "reservations.html", context)



from django.urls import path

from reservations import views

urlpatterns = [
    path("crud_reservation", views.crud_reservation, name="crud_reservation"),
    path("crud_reservation/<int:pk>", views.crud_reservation, name="crud_reservation"),
    path("reservations", views.reservations, name="reservations"),
    path("delete_reservation/<int:pk>", views.delete_reservation, name="delete_reservation")
]



from django.urls import path

from vehicles import views

urlpatterns = [
    path("", views.vehicles, name="vehicles"),
    path("vehicle/<str:pk>", views.vehicle, name="vehicle"),
    path("vehicles", views.vehicles, name="vehicles"),
    path("bulk_create_vehicles", views.bulk_create_vehicles, name="bulk_create_vehicles"),
]



from django import forms

class ReservationForm(forms.Form):
    vehicle = forms.CharField(max_length=12)
    client_name = forms.CharField(max_length=100)
    departure_dt = forms.DateTimeField()
    return_dt = forms.DateTimeField()
    details = forms.CharField()



from django.db import models

from vehicles.models import Vehicle

class Reservation(models.Model):
    vehicle = models.ForeignKey(Vehicle, on_delete=models.CASCADE)
    client_name = models.CharField(max_length=100)
    departure_dt = models.DateTimeField()
    return_dt = models.DateTimeField()
    details = models.TextField(blank=True)



import os
from pathlib import Path

MANAGE_PATH = Path(__file__).parent.joinpath("manage.py")

def runserver():
    os.system(f"python {MANAGE_PATH} makemigrations reservations")
    os.system(f"python {MANAGE_PATH} makemigrations vehicles")
    os.system(f"python {MANAGE_PATH} migrate")
    os.system(f"python {MANAGE_PATH} runserver 0.0.0.0:8000")

def create_admin():
    os.system(f"python {MANAGE_PATH} createsuperuser")
    exit()

if __name__ == "__main__":
    import argparse

    parser = argparse.ArgumentParser()
    parser.add_argument("--create-admin", action="store_true", help="create the admin user")
    args = parser.parse_args()

    if args.create_admin:
        create_admin()
    else:
        runserver()



import os
from importlib.resources import files
import atexit

from setuptools import setup
from setuptools.command.install import install

import re
VERSIONFILE="salaora/_version.py"
verstrline = open(VERSIONFILE, "rt").read()
VSRE = r"^__version__ = ['\"]([^'\"]*)['\"]"
mo = re.search(VSRE, verstrline, re.M)
if mo:
    verstr = mo.group(1)
else:
    raise RuntimeError("Unable to find version string in %s." % (VERSIONFILE,))

def install_executables():
    if os.name == "nt":
        base_dir = os.path.join(os.environ['USERPROFILE'], 'Desktop')
        run_file = os.path.join(base_dir, "run_salaora.bat")
        with open(run_file, "w", encoding="utf8") as fout:
            fout.write("python -m salaora")

        upgrade_file = os.path.join(base_dir, "upgrade_salaora.bat")
        with open(upgrade_file, "w", encoding="utf8") as fout:
            fout.write("pip install --upgrade salaora")

class new_install(install):
    def __init__(self, *args, **kwargs):
        super(new_install, self).__init__(*args, **kwargs)
        atexit.register(install_executables)

setup(
    cmdclass = {
        "install": new_install
    },
    version = verstr
)



from io import StringIO
import csv

from django.db import models

from salaora import now

class Vehicle(models.Model):
    COLOR_CHOICES = [
        ("Blue", "Blue"),
        ("Black", "Black"),
        ("White", "White"),
        ("Gray", "Gray")
    ]
    VEHICLE_TYPE_CHOICES = [
        ("Car", "Car"),
        ("Jeep", "Jeep"),
        ("ATV", "ATV"),
        ("Scooter", "Scooter")
    ]
    ENGINE_TYPE_CHOICES = [
        ("Gas", "Gas"),
        ("Diesel", "Diesel")
    ]

    @property
    def is_available(self, from_dt=None, to_dt=None):
        for reservation in self.reservation_set.all():
            if reservation.departure_dt < now < reservation.return_dt:
                return False
        return True

    model = models.CharField(max_length=100)
    plate = models.CharField(max_length=12, primary_key=True)
    color = models.CharField(max_length=20, choices=COLOR_CHOICES)
    vehicle_type = models.CharField(max_length=10, choices=VEHICLE_TYPE_CHOICES)
    engine_type = models.CharField(max_length=10, choices=ENGINE_TYPE_CHOICES)
    price = models.FloatField(blank=True)
    details = models.TextField(blank=True)
    image = models.FileField(upload_to="vehicle_images/", blank=True, default="defaults/no_car.png")

def populate_vehicles_from_csv_string(csv_data):
    f = StringIO(csv_data)
    reader = csv.reader(f)
    next(reader)

    for row in reader:
        print(row)
        try:
            model = row[0]
            plate = row[1]
            color = row[2]
            vehicle_type = row[3]
            engine_type = row[4]
            price = float(row[5])
            details = row[6]

            vehicle = Vehicle(
                model = model,
                plate = plate,
                color = color,
                vehicle_type = vehicle_type,
                engine_type = engine_type,
                price = price,
                details = details
            )
            vehicle.full_clean()
            vehicle.save()
        except Exception as e:
            print(e)
